import math
#from python_matrix_operations import *
import ctypes
import os
import sys
import numpy as np
import elasticity_toolbox as elast

#---------------------------------------------------------------
# Wooden bilayer, https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6191116/#pone.0205607.ref015
#--- define indicator function
# x[0] : y1-component -1/2 to 1/2
# x[1] : y2-component -1/2 to 1/2
# x[2] : x3-component range -1/2 to 1/2
#--- define indicator function

# Parameters of the model
# -- ratio between upper layer / thickness, thickness [m]
param_width = 1
param_r = 0.3
param_h = 0.0053
# -- moisture content in the flat state and target state [%]
param_omega_flat = 17.17547062
param_omega_target = 8.959564147
param_theta = 0
#
#
#
delta_omega=param_omega_target-param_omega_flat
# moisture content for material law
omega=param_omega_target

# --- Material properties from https://www.ncbi.nlm.nih.gov/pmc/articles/PMC6191116/#pone.0205607.ref015
# --- for European beech, moisture content omega = 15%
# --- L=direction orthogonal to layering and fibres = orthogonal to wood stem cross-section
# --- T=tangential zu layering
# --- R=orthogonal zu layering
# --- in MPa
# --- Properties are defined by affine function in dependence of moisture content omega via property = b_0+b_1 \omega
# --- coefficients of affine function are contained in the following array 
# --- data taken from http://dx.doi.org/10.1016/j.cma.2014.10.031

properties_coefficients=np.array([
    # [b_0, b_1]    
[2565.6,-59.7], # E_R [MPa]
[885.4, -23.4], # E_T [MPa]
[17136.7,-282.4], # E_L [MPa]
[667.8, -15.19], # G_RT [MPa]
[1482, -15.26], # G_RL [MPa]
[1100, -17.72], # G_TL [MPa]
[0.2933, -0.001012], # nu_TR [1]
[0.383, -0.008722], # nu_LR [1]
[0.3368, -0.009071] # nu_LT [1]
])
# Compute actual properties
#E_R=1500
E_R = properties_coefficients[0,0]+properties_coefficients[0,1]*omega
# E_T=450
E_T = properties_coefficients[1,0]+properties_coefficients[1,1]*omega
#E_L=12000
E_L = properties_coefficients[2,0]+properties_coefficients[2,1]*omega
# G_RT=400
G_RT = properties_coefficients[3,0]+properties_coefficients[3,1]*omega
# G_RL=1200
G_LR = properties_coefficients[4,0]+properties_coefficients[4,1]*omega
# G_TL=800
G_LT  = properties_coefficients[5,0]+properties_coefficients[5,1]*omega
# nu_TR=0.28
nu_TR  = properties_coefficients[6,0]+properties_coefficients[6,1]*omega
# nu_LR=0.2125
nu_LR  = properties_coefficients[7,0]+properties_coefficients[7,1]*omega
# nu_LT=0.175
nu_LT  = properties_coefficients[8,0]+properties_coefficients[8,1]*omega
#
nu_TL=nu_LT*E_T/E_L
nu_RT=nu_TR*E_R/E_T
nu_RL=nu_LR*E_R/E_L

# --- differential swelling strain
# --- relation to swelling strain eps: eps=alpha* delta_omega with delta_omega = change of water content in %
alpha_L=0.00011 # PLOS paper
alpha_R=0.00191 # PLOS paper
alpha_T=0.00462 # PLOS paper
# Umrechnen
#alpha_L=(1-1/(1+delta_omega*alpha_L))/delta_omega
#alpha_R=(1-1/(1+delta_omega*alpha_R))/delta_omega
#alpha_T=(1-1/(1+delta_omega*alpha_T))/delta_omega
# --- define geometry

def indicatorFunction(x):
    factor=1
    if (x[2]>=(0.5-param_r)):
        return 1
    else :
        if (x[0]>(0.5-param_width)) and (x[2]>=0):
            return 1
        else :
            return 2

# --- Number of material phases
Phases=3

# --- PHASE 1
# y_1-direction: L
# y_2-direction: T
# x_3-direction: R
# phase1_type="orthotropic"
# materialParameters_phase1 = [E_L,E_T,E_R,G_TL,G_RT,G_RL,nu_LT,nu_LR,nu_TR]
phase1_type="general_anisotropic"
[E_1,E_2,E_3]=[E_L,E_T,E_R]
[nu_12,nu_13,nu_23]=[nu_LT,nu_LR,nu_TR]
[nu_21,nu_31,nu_32]=[nu_TL,nu_RL,nu_RT]
[G_12,G_31,G_23]=[G_LT,G_LR,G_RT]
compliance_S=np.array([[1/E_1,         -nu_21/E_2,     -nu_31/E_3,   0.0,         0.0,  0.0],
                       [-nu_12/E_1,      1/E_2,        -nu_32/E_3,   0.0,         0.0,  0.0],
                       [-nu_13/E_1,     -nu_23/E_2,         1/E_3,   0.0,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   1/G_23,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         1/G_31,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         0.0,  1/G_12]]);
materialParameters_phase1 = compliance_S
def prestrain_phase1(x):
    return [[1/param_h*delta_omega*alpha_L, 0, 0], [0,1/param_h*delta_omega*alpha_T,0], [0,0,1/param_h*delta_omega*alpha_R]]

# --- PHASE 2
# y_1-direction: R
# y_2-direction: L
# x_3-direction: T
phase2_type="general_anisotropic"
[E_1,E_2,E_3]=[E_R,E_L,E_T]
[nu_12,nu_13,nu_23]=[nu_RL,nu_RT,nu_LT]
[nu_21,nu_31,nu_32]=[nu_LR,nu_TR,nu_TL]
[G_12,G_31,G_23]=[G_LR,G_RT,G_LT]
compliance_S=np.array([[1/E_1,         -nu_21/E_2,     -nu_31/E_3,   0.0,         0.0,  0.0],
                       [-nu_12/E_1,      1/E_2,        -nu_32/E_3,   0.0,         0.0,  0.0],
                       [-nu_13/E_1,     -nu_23/E_2,         1/E_3,   0.0,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   1/G_23,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         1/G_31,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         0.0,  1/G_12]]);
materialParameters_phase2 = compliance_S
#phase2_type="orthotropic"
#materialParameters_phase2 = [E_R, E_L, E_T,G_RL, G_TL, G_RT,nu_LR, nu_TR,nu_LT]
def prestrain_phase2(x):
    return [[1/param_h*delta_omega*alpha_R, 0, 0], [0,1/param_h*delta_omega*alpha_L,0], [0,0,1/param_h*delta_omega*alpha_T]]

# --- PHASE 1
# y_1-direction: L
# y_2-direction: T
# x_3-direction: R
# phase1_type="orthotropic"
# materialParameters_phase1 = [E_L,E_T,E_R,G_TL,G_RT,G_RL,nu_LT,nu_LR,nu_TR]
phase3_type="general_anisotropic"
[E_1,E_2,E_3]=[E_T,E_L,E_R]
[nu_12,nu_13,nu_23]=[nu_TL,nu_TR,nu_LR]
[nu_21,nu_31,nu_32]=[nu_LT,nu_RT,nu_RL]
[G_12,G_31,G_23]=[G_LT,G_RT,G_LR]
compliance_S=np.array([[1/E_1,         -nu_21/E_2,     -nu_31/E_3,   0.0,         0.0,  0.0],
                       [-nu_12/E_1,      1/E_2,        -nu_32/E_3,   0.0,         0.0,  0.0],
                       [-nu_13/E_1,     -nu_23/E_2,         1/E_3,   0.0,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   1/G_23,         0.0,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         1/G_31,  0.0],
                       [0.0,                0.0,              0.0,   0.0,         0.0,  1/G_12]]);
materialParameters_phase3 = compliance_S
def prestrain_phase3(x):
    return [[1/param_h*delta_omega*alpha_T, 0, 0], [0,1/param_h*delta_omega*alpha_L,0], [0,0,1/param_h*delta_omega*alpha_R]]
